with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions,
  terms;

procedure prism1 is

  type cable_rec is
    record
      state : integer := 0;
      glob  : terms.imat1 (1 .. 2);
    end record;

  type cable_vec is array (integer range <>) of cable_rec;

  type cable_ptr is access cable_vec;

  nj,nn,ne,state                   : integer;
  area0,young0,area,young,value,fz : long_float;
  xo,yo,zo                         : terms.fmat1_ptr;
  cable                            : cable_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics,
      ada.numerics.long_elementary_functions;

    e,glob           : integer;
    radius,height,aj : long_float;
    io               : file_type;

  begin
    open(io, in_file, "prism1.txt");

    -- nj, radius, height, fz
    scan(io);
    get(io,nj);
    get(io,radius);
    get(io,height);
    get(io,fz);
    skip_line(io);

    -- area0, young0
    scan(io);
    get(io,area0);
    get(io,young0);
    skip_line(io);

    -- area, young, state, value
    scan(io);
    get(io,area);
    get(io,young);
    get(io,state);
    get(io,value);
    skip_line(io);

    close(io);

    -- nn
    nn := 2 * nj;
  
    -- ne
    ne := 4 * nj;

    -- nodes
    xo := new terms.fmat1 (1 .. nn);
    yo := new terms.fmat1 (1 .. nn);
    zo := new terms.fmat1 (1 .. nn);

    glob := 0;

    for i in 0 .. 1 loop
      for j in 0 .. nj - 1 loop
        glob := glob + 1;
        aj := ((long_float(j + j) + 1.0) / long_float(nj) - 0.5) * pi;
        xo(glob) := radius * cos(aj);
        yo(glob) := radius * sin(aj);
        zo(glob) := height * long_float(i);
      end loop;
    end loop;

    -- cable elements
    cable := new cable_vec (1 .. ne);

    e := 0;

    -- face elements
    for i in 0 .. 1 loop
      for j in 1 .. nj loop
        e := e + 1;
        cable(e).glob(1) := nj * i + j;
        cable(e).glob(2) := nj * i + j mod nj + 1;
      end loop;
    end loop;

    -- vertical elements
    for j in 1 .. nj loop
      e := e + 1;
      cable(e).glob(1) := j;
      cable(e).glob(2) := nj + j;
    end loop;

    -- diagonal elements
    for j in 1 .. nj loop
      e := e + 1;
      cable(e).glob(1) := j mod nj + 1;
      cable(e).glob(2) := nj + j;
      cable(e).state := state;
    end loop;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist : integer;
    lstol,mntol,mnref      : long_float;
    io                     : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    mntol := 1.0E-06;
    mnref := 1.0E+00;
    maxminor := 30 * nn;
    maxlist := 3;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls             mntol             mnref  maxminor   maxlist");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,mntol,2,6,3);
    put(io,"     ");
    put(io,mnref,2,6,3);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);
    for glob in 1 .. nn loop
      put(io,glob,5);
      put(io,"     ");
      put(io,xo(glob),2,6,3);
      put(io,"     ");
      put(io,yo(glob),2,6,3);
      put(io,"     ");
      put(io,zo(glob),2,6,3);
      new_line(io);
    end loop;

    -- cable elements
    new_line(io);
    new_line(io);
    put(io," elem      node      node              area             young");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    new_line(io);
    for e in cable'range loop
      put(io,e,5);
      put(io,"     ");
      put(io,cable(e).glob(1),5);
      put(io,"     ");
      put(io,cable(e).glob(2),5);
      if cable(e).state = state then
        put(io,"     ");
        put(io,area,2,6,3);
        put(io,"     ");
        put(io,young,2,6,3);
        new_line(io);
      else
        put(io,"     ");
        put(io,area0,2,6,3);
        put(io,"     ");
        put(io,young0,2,6,3);
        new_line(io);
      end if;
    end loop;

    -- membrane elements
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         stiffness           poisson");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nj,5);
    new_line(io);

    new_line(io);
    for glob in 1 .. nj loop
      put(io,glob,5);
      put(io,"         3      0.000000E+00");
      new_line(io);
    end loop;

    -- set cable element state
    new_line(io);
    new_line(io);

    case state is
      when 1 =>
        put(io," elem     state               cut");
        new_line(io);

      when 2 =>
        put(io," elem     state           tension");
        new_line(io);

      when others =>
        null;
    end case;

    put(io,":");
    new_line(io);
    put(io,nj,5);
    new_line(io);

    new_line(io);
    for e in cable'range loop
      if cable(e).state = state then
        put(io,e,5);
        put(io,"     ");
        put(io,state,5);
        put(io,"     ");
        put(io,value,2,6,3);
        new_line(io);
      end if;
    end loop;

    -- loading
    new_line(io);
    new_line(io);
    put(io," node      axis             force");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nj,5);
    new_line(io);

    new_line(io);
    for glob in nj + 1 .. nj + nj loop
      put(io,glob,5);
      put(io,"         3     ");
      put(io,fz,2,6,3);
      new_line(io);
    end loop;

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    io   : file_type;

  begin
    create(io, out_file, "prism1.scr");

    -- autocad
    put(io,"osmode 0");
    new_line(io);

    put(io,"layer make Geometry ");
    new_line(io);

    -- cable elements
    for e in cable'range loop
      if cable(e).state /= state then
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := cable(e).glob(loc);
          put(io,xo(glob),1,8,3);
          put(io,",");
          put(io,yo(glob),1,8,3);
          put(io,",");
          put(io,zo(glob),1,8,3);
          put(io," ");
        end loop;
        new_line(io);
      end if;
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    for e in cable'range loop
      if cable(e).state = state then
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := cable(e).glob(loc);
          put(io,xo(glob),1,8,3);
          put(io,",");
          put(io,yo(glob),1,8,3);
          put(io,",");
          put(io,zo(glob),1,8,3);
          put(io," ");
        end loop;
        new_line(io);
      end if;
    end loop;

    -- autocad
    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end prism1;
