with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics.long_elementary_functions,
  terms;

procedure stella is

  type cable_rec is
    record
      state : integer := 0;
      value : long_float := 0.0;
      glob  : terms.imat1 (1 .. 2);
    end record;

  type cable_vec is array (integer range <>) of cable_rec;

  type cable_ptr is access cable_vec;

  state                   : integer;
  area0,young0,area,young : long_float;
  xo,yo,zo                : terms.fmat1_ptr;
  cable                   : cable_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics.long_elementary_functions;

    nk,e  : integer;
    s,r,h : long_float;
    io    : file_type;

  begin
    -- cable elements
    cable := new cable_vec (1 .. 24);

    cable(1).glob(1) := 4;
    cable(1).glob(2) := 1;
    cable(2).glob(1) := 4;
    cable(2).glob(2) := 10;
    cable(3).glob(1) := 4;
    cable(3).glob(2) := 11;

    cable(4).glob(1) := 5;
    cable(4).glob(2) := 2;
    cable(5).glob(1) := 5;
    cable(5).glob(2) := 12;
    cable(6).glob(1) := 5;
    cable(6).glob(2) := 10;

    cable(7).glob(1) := 6;
    cable(7).glob(2) := 3;
    cable(8).glob(1) := 6;
    cable(8).glob(2) := 11;
    cable(9).glob(1) := 6;
    cable(9).glob(2) := 12;

    cable(10).glob(1) := 7;
    cable(10).glob(2) := 10;
    cable(11).glob(1) := 7;
    cable(11).glob(2) := 2;
    cable(12).glob(1) := 7;
    cable(12).glob(2) := 1;

    cable(13).glob(1) := 8;
    cable(13).glob(2) := 11;
    cable(14).glob(1) := 8;
    cable(14).glob(2) := 1;
    cable(15).glob(1) := 8;
    cable(15).glob(2) := 3;

    cable(16).glob(1) := 9;
    cable(16).glob(2) := 12;
    cable(17).glob(1) := 9;
    cable(17).glob(2) := 3;
    cable(18).glob(1) := 9;
    cable(18).glob(2) := 2;

    cable(19).glob(1) := 1;
    cable(19).glob(2) := 2;
    cable(20).glob(1) := 2;
    cable(20).glob(2) := 3;
    cable(21).glob(1) := 3;
    cable(21).glob(2) := 1;

    cable(22).glob(1) := 10;
    cable(22).glob(2) := 11;
    cable(23).glob(1) := 11;
    cable(23).glob(2) := 12;
    cable(24).glob(1) := 12;
    cable(24).glob(2) := 10;

    open(io, in_file, "stella.txt");

    -- s
    scan(io);
    get(io,s);
    skip_line(io);

    -- area0, young0
    scan(io);
    get(io,area0);
    get(io,young0);

    -- area, young, state
    scan(io);
    get(io,area);
    get(io,young);
    get(io,state);
    skip_line(io);

    -- value
    scan(io);
    get(io,nk);
    skip_line(io);

    skip_line(io);
    for k in 1 .. nk loop
      get(io,e);
      get(io,cable(e).value);
      skip_line(io);
      cable(e).state := state;
    end loop;

    close(io);

    -- nodes
    xo := new terms.fmat1 (1 .. 12);
    yo := new terms.fmat1 (1 .. 12);
    zo := new terms.fmat1 (1 .. 12);

    r := s / sqrt(3.0);
    h := s / sqrt(6.0);

    xo(1) := -s / 2.0;
    yo(1) := -r / 2.0;
    zo(1) := h;

    xo(2) := s / 2.0;
    yo(2) := -r / 2.0;
    zo(2) := h;

    xo(3) := 0.0;
    yo(3) := r;
    zo(3) := h;

    xo(4) := 0.0;
    yo(4) := -2.0 * r;
    zo(4) := h;

    xo(5) := s;
    yo(5) := r;
    zo(5) := h;

    xo(6) := -s;
    yo(6) := r;
    zo(6) := h;

    xo(7) := s;
    yo(7) := -r;
    zo(7) := -h;

    xo(8) := -s;
    yo(8) := -r;
    zo(8) := -h;

    xo(9) := 0.0;
    yo(9) := 2.0 * r;
    zo(9) := -h;

    xo(10) := 0.0;
    yo(10) := -r;
    zo(10) := -h;

    xo(11) := -s / 2.0;
    yo(11) := r / 2.0;
    zo(11) := -h;

    xo(12) := s / 2.0;
    yo(12) := r / 2.0;
    zo(12) := -h;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist : integer;
    lstol,mntol,mnref      : long_float;
    io                     : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    mntol := 1.0E-06;
    mnref := 1.0E+00;
    maxminor := 360;
    maxlist := 3;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls             mntol             mnref  maxminor   maxlist");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,mntol,2,6,3);
    put(io,"     ");
    put(io,mnref,2,6,3);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"   12");
    new_line(io);

    new_line(io);
    for glob in 1 .. 12 loop
      put(io,glob,5);
      put(io,"     ");
      put(io,xo(glob),2,6,3);
      put(io,"     ");
      put(io,yo(glob),2,6,3);
      put(io,"     ");
      put(io,zo(glob),2,6,3);
      new_line(io);
    end loop;

    -- cable elements
    new_line(io);
    new_line(io);
    put(io," elem      node      node              area             young");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,cable'last,5);
    new_line(io);

    new_line(io);
    for e in cable'range loop
      put(io,e,5);
      put(io,"     ");
      put(io,cable(e).glob(1),5);
      put(io,"     ");
      put(io,cable(e).glob(2),5);
      if cable(e).state = state then
        put(io,"     ");
        put(io,area,2,6,3);
        put(io,"     ");
        put(io,young,2,6,3);
        new_line(io);
      else
        put(io,"     ");
        put(io,area0,2,6,3);
        put(io,"     ");
        put(io,young0,2,6,3);
        new_line(io);
      end if;
    end loop;

    -- membrane elements
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         stiffness           poisson");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    6");
    new_line(io);

    new_line(io);
    put(io,"    1         1      0.000000E+00");
    new_line(io);
    put(io,"    1         2      0.000000E+00");
    new_line(io);
    put(io,"    1         3      0.000000E+00");
    new_line(io);
    put(io,"    2         2      0.000000E+00");
    new_line(io);
    put(io,"    2         3      0.000000E+00");
    new_line(io);
    put(io,"    3         3      0.000000E+00");
    new_line(io);

    -- set cable element state
    new_line(io);
    new_line(io);

    case state is
      when 1 =>
        put(io," elem     state               cut");
        new_line(io);

      when 2 =>
        put(io," elem     state           tension");
        new_line(io);

      when others =>
        null;
    end case;

    put(io,":");
    new_line(io);
    put(io,"    6");
    new_line(io);

    new_line(io);
    for e in cable'range loop
      if cable(e).state = state then
        put(io,e,5);
        put(io,"     ");
        put(io,state,5);
        put(io,"     ");
        put(io,cable(e).value,2,6,3);
        new_line(io);
      end if;
    end loop;

    -- loading
    new_line(io);
    new_line(io);
    put(io," node      axis             force");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    io   : file_type;

  begin
    create(io, out_file, "stella.scr");

    -- autocad
    put(io,"osmode 0");
    new_line(io);

    put(io,"layer make Geometry ");
    new_line(io);

    -- cable elements
    for e in cable'range loop
      if cable(e).state /= state then
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := cable(e).glob(loc);
          put(io,xo(glob),1,8,3);
          put(io,",");
          put(io,yo(glob),1,8,3);
          put(io,",");
          put(io,zo(glob),1,8,3);
          put(io," ");
        end loop;
        new_line(io);
      end if;
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    for e in cable'range loop
      if cable(e).state = state then
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := cable(e).glob(loc);
          put(io,xo(glob),1,8,3);
          put(io,",");
          put(io,yo(glob),1,8,3);
          put(io,",");
          put(io,zo(glob),1,8,3);
          put(io," ");
        end loop;
        new_line(io);
      end if;
    end loop;

    -- autocad
    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end stella;
